package com.example.custom_track

import FlashingPointOverlay
import android.Manifest
import android.content.pm.PackageManager
import android.location.Location
import android.os.Bundle
import android.os.Handler
import android.os.Looper
import android.util.Log
import android.widget.Toast
import androidx.appcompat.app.AppCompatActivity
import androidx.core.app.ActivityCompat
import com.google.android.gms.location.*
import com.google.gson.Gson
import com.google.gson.JsonSyntaxException
import com.google.gson.reflect.TypeToken
import org.osmdroid.config.Configuration
import org.osmdroid.util.GeoPoint
import org.osmdroid.views.MapView
import org.osmdroid.views.overlay.Marker
import org.osmdroid.views.overlay.Polygon
import org.osmdroid.views.overlay.Polyline
import java.util.concurrent.TimeUnit
import okhttp3.MediaType.Companion.toMediaType
import okhttp3.OkHttpClient
import okhttp3.RequestBody
import okhttp3.Request
import org.json.JSONArray
import org.json.JSONException
import java.io.IOException

class MapActivity : AppCompatActivity() {

    private lateinit var mapView: MapView
    private lateinit var fusedLocationClient: FusedLocationProviderClient
    private lateinit var locationRequest: LocationRequest
    private lateinit var locationCallback: LocationCallback
    private var currentLocationMarker: Marker? = null
    private val periodicHandler = Handler()
    private lateinit var periodicTask: Runnable

    private val client = OkHttpClient.Builder()
        .callTimeout(15, TimeUnit.SECONDS)
        .build()

    companion object {
        private const val PERMISSION_REQUEST_CODE = 1
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_map)

        Configuration.getInstance().userAgentValue = "YourAppName/1.0 (https://www.yourwebsite.com)"

        // Initialize MapView
        mapView = findViewById(R.id.map)
        mapView.setMultiTouchControls(true)

        // Initialize FusedLocationProviderClient
        fusedLocationClient = LocationServices.getFusedLocationProviderClient(this)

        // Create LocationRequest
        locationRequest = LocationRequest.create().apply {
            interval = 2000L // 2 seconds
            fastestInterval = 1000L // 1 second
            priority = LocationRequest.PRIORITY_HIGH_ACCURACY
        }

        // Define the LocationCallback
        locationCallback = object : LocationCallback() {
            override fun onLocationResult(locationResult: LocationResult) {
                val location = locationResult.lastLocation
                if (location != null) {
                    updateCurrentLocation(location)
                }
            }
        }

        // Check permissions
        checkPermissions()

        // Get spatial data from intent
        val spatialJson = intent.getStringExtra("SPATIAL_VALUES")
        val operationSiteType = intent.getStringExtra("OPERATION_SITE_TYPE")
        val assignedEmployeeID = intent.getStringExtra("ASSIGNED_EMPLOYEE_ID")
        val employeeHRID = intent.getStringExtra("EMPLOYEE_HR_ID")
//        Log.d("from backend spatialJson","$spatialJson")
//        Log.d("from backend operationSiteType","$operationSiteType")
//        Log.d("from backend assignedEmployeeID","$assignedEmployeeID")
//        Log.d("from backend employeeHRID","$employeeHRID")

        try {

            val actualJson = spatialJson.let { Gson().fromJson(it, String::class.java) }
            if (actualJson == null) {
                Log.e("MapActivity", "Parsed JSON is null")
                Toast.makeText(this, "Invalid spatial data format.", Toast.LENGTH_LONG).show()
                return
            }

//            val actualJson = Gson().fromJson(spatialJson, String::class.java)

            when (operationSiteType) {
                "LOCATION" -> {

                    val pointData: List<Double> = Gson().fromJson(actualJson, object : TypeToken<List<Double>>() {}.type)
                    val geoPoint = GeoPoint(pointData[1], pointData[0])

                    val marker = Marker(mapView).apply {
                        position = geoPoint
                        title = "Location"
                    }
                    marker.setOnMarkerClickListener { _, _ ->
                        Toast.makeText(this, "Assigned Employee ID: $assignedEmployeeID\nEmployee HR ID: $employeeHRID", Toast.LENGTH_LONG).show()
                        true
                    }
                    mapView.overlays.add(marker)

                    mapView.controller.apply {
                        setZoom(10.0)
                        setCenter(geoPoint)
                    }
                }

                "ROAD" -> {
                    val roadData: List<List<Double>> = Gson().fromJson(actualJson, object : TypeToken<List<List<Double>>>() {}.type)
                    val geoPoints = roadData.map { GeoPoint(it[1], it[0]) }

                    val polyline = Polyline().apply {
                        setPoints(geoPoints)
                        title = "Road"
                    }
                    polyline.setOnClickListener { _, _, _ ->
                        Toast.makeText(this, "Assigned Employee ID: $assignedEmployeeID", Toast.LENGTH_LONG).show()
                        true
                    }
                    mapView.overlays.add(polyline)

                    if (geoPoints.isNotEmpty()) {
                        mapView.controller.apply {
                            setZoom(10.0)
                            setCenter(geoPoints[0])
                        }
                    }
                }

                "AREA" -> {
                    val areaData: List<List<Double>> = Gson().fromJson(actualJson, object : TypeToken<List<List<Double>>>() {}.type)
                    val geoPoints = areaData.map { GeoPoint(it[1], it[0]) }

                    val polygon = Polygon().apply {
                        setPoints(geoPoints)
                        title = "Area"
                    }
                    polygon.setOnClickListener { _, _, _ ->
                        Toast.makeText(this, "Assigned Employee ID: $assignedEmployeeID\nEmployee HR ID: $employeeHRID", Toast.LENGTH_LONG).show()
                        true
                    }
                    mapView.overlays.add(polygon)

                    if (geoPoints.isNotEmpty()) {
                        mapView.controller.apply {
                            setZoom(10.0)
                            setCenter(geoPoints[0])
                        }
                    }
                }

                else -> {
                    Log.e("MapActivity", "Unknown operation site type: $operationSiteType")
                }
            }

        } catch (e: JsonSyntaxException) {
            Log.e("MapActivity", "Invalid JSON format: $spatialJson", e)
        }

        startPeriodicDataPost()
    }

    private fun checkPermissions() {
        if (ActivityCompat.checkSelfPermission(this, Manifest.permission.ACCESS_FINE_LOCATION) != PackageManager.PERMISSION_GRANTED) {
            ActivityCompat.requestPermissions(this, arrayOf(Manifest.permission.ACCESS_FINE_LOCATION), PERMISSION_REQUEST_CODE)
        } else {
            startLocationUpdates()
        }
    }

    override fun onRequestPermissionsResult(requestCode: Int, permissions: Array<out String>, grantResults: IntArray) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults)
        if (requestCode == PERMISSION_REQUEST_CODE) {
            if (grantResults.isNotEmpty() && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
                startLocationUpdates()
            } else {
                Toast.makeText(this, "Permission denied for location access", Toast.LENGTH_SHORT).show()
            }
        }
    }
    private fun startLocationUpdates() {
        if (ActivityCompat.checkSelfPermission(this, Manifest.permission.ACCESS_FINE_LOCATION) == PackageManager.PERMISSION_GRANTED) {
            fusedLocationClient.requestLocationUpdates(locationRequest, locationCallback, mainLooper)
        } else {
            Toast.makeText(this, "Location permission not granted. Unable to start location updates.", Toast.LENGTH_SHORT).show()
        }
    }


    private fun updateCurrentLocation(location: Location) {
        try {
            val geoPoint = GeoPoint(location.latitude, location.longitude)

            currentLocationMarker?.let {
                mapView.overlays.remove(it)
            }

            currentLocationMarker = null // Clear the marker reference

            val flashingOverlay = FlashingPointOverlay(this, geoPoint)
            mapView.overlays.add(flashingOverlay)

            mapView.overlays.add(flashingOverlay)

            mapView.controller.apply {
                setZoom(8.0)
                setCenter(geoPoint)
            }

// Refresh the map to display the animation
            mapView.invalidate()


        } catch (e: Exception) {
            Log.e("MapActivity", "Error updating current location", e)
        }
    }

    private fun startPeriodicDataPost() {
        val spatialValue = intent.getStringExtra("SPATIAL_VALUES")
        val operationSiteType = intent.getStringExtra("OPERATION_SITE_TYPE")
        val assignedEmployeeID = intent.getStringExtra("ASSIGNED_EMPLOYEE_ID")
        val employeeHRID = intent.getStringExtra("EMPLOYEE_HR_ID")
        val resourcePlanID = intent.getStringExtra("RESOURCE_PLAN_ID")

        periodicTask = object : Runnable {
            override fun run() {
                val currentLocation = currentLocationMarker?.position
//                Log.d("current",currentLocation?.latitude.toString())
//                Log.d("current",currentLocation?.longitude.toString())
                if (currentLocation != null) {
                    postDataToServer(
                        currentLocation.latitude,
                        currentLocation.longitude,
                        resourcePlanID,
                        assignedEmployeeID,
                        employeeHRID
                    )
                }
                periodicHandler.postDelayed(this, 150)
            }
        }
        periodicHandler.post(periodicTask)
    }

    private fun postDataToServer(latitude: Double, longitude: Double, resourcePlanID: String?, assignedEmployeeID: String?, employeeHRID: String?) {
        val url = "http://192.168.50.166:8096/gis/gazetteer"

        val jsonBody = """
            {
                "latitude": $latitude,
                "longitude": $longitude,
                "resourcePlanID": "$resourcePlanID",
                "assignedEmployeeID": "$assignedEmployeeID",
                "employeeHRID": "$employeeHRID"
            }
        """.trimIndent()

        val requestBody = RequestBody.create("application/json; charset=utf-8".toMediaType(), jsonBody)

        val request = Request.Builder()
            .url(url)
            .post(requestBody)
            .build()

        client.newCall(request).enqueue(object : okhttp3.Callback {
            override fun onFailure(call: okhttp3.Call, e: IOException) {
                Log.e("MapActivity", "Failed to post data: ${e.message}")
            }

            override fun onResponse(call: okhttp3.Call, response: okhttp3.Response) {
                if (response.isSuccessful) {
                    Log.d("MapActivity", "Data posted successfully: ${response.body?.string()}")
                } else {
                    Log.e("MapActivity", "Failed to post data: ${response.code}")
                }
            }
        })
    }

    override fun onPause() {
        super.onPause()
        fusedLocationClient.removeLocationUpdates(locationCallback)
        periodicHandler.removeCallbacks(periodicTask)
    }
}
